package pc.feqhiat.db;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

import java.util.ArrayList;
import java.util.List;
import pc.feqhiat.R;
import pc.feqhiat.model.MainVO;

public class DBH_Main extends SQLiteOpenHelper {

	private static final int DATABASE_VERSION = 1;
	private static final String DATABASE_NAME = "feqhiat";
	////////// Tables /////////////////////
	private static final String TABLE_ITEMS = "item";
	private static final String TABLE_BOOKS = "book";
	private static final String TABLE_CHAPTERS = "chapter";
	private static final String TABLE_FAQIH = "faqih";
    private static final String TABLE_PARAM = "param";
	/////////////// Item Table Fields ///////////////////////
	private static final String KEY_ITEMID = "item_id"; // primary key
	private static final String KEY_ITEMDESC = "item_name";
	////// COMMON COLUMNS NAME /////////////
	private static final String KEY_UPDATE = "update_time";
    private static final String KEY_FILEPATH = "file_path";
	/////////////// Book Table Fields ///////////////////////
	private static final String KEY_BOOKID = "book_id"; // primary key
	private static final String KEY_BOOKDESC = "book_name";
	/////////////// CHAPTER Table Fields ///////////////////////
	private static final String KEY_CHAPTERID = "chapter_id"; // primary key
	private static final String KEY_CHAPTERDESC = "chapter_name";
	/////////////// FAQIH Table Fields ///////////////////////
	private static final String KEY_FAQIHID = "faqih_id"; // primary key
	private static final String KEY_FAQIHNAME = "faqih_name";
    /////////////// Param Table Fields ///////////////////////
    private static final String KEY_PARAMID = "param_id"; // primary key
    private static final String KEY_PARAMNAME = "name";
    private static final String KEY_PARAMVALUE = "value";

    /////////// String for item table //////////////
	String CREATE_ITEM_TABLE = "CREATE TABLE " + TABLE_ITEMS + "("
			+ KEY_ITEMID + " INTEGER PRIMARY KEY," + KEY_ITEMDESC + " TEXT,"  + KEY_UPDATE + " INTEGER,"
			+ KEY_FILEPATH + " TEXT" + ")";
    /////////// String for book table //////////////
    String CREATE_BOOK_TABLE = "CREATE TABLE " + TABLE_BOOKS + "("
            + KEY_BOOKID + " INTEGER PRIMARY KEY," + KEY_BOOKDESC + " TEXT,"  + KEY_UPDATE + " INTEGER,"
            + KEY_FILEPATH + " TEXT" + ")";
    /////////// String for chapter table //////////////
    String CREATE_CHAPTER_TABLE = "CREATE TABLE " + TABLE_CHAPTERS + "("
            + KEY_CHAPTERID + " INTEGER PRIMARY KEY, "+ KEY_BOOKID + " INTEGER, "  + KEY_CHAPTERDESC + " TEXT,"  + KEY_UPDATE + " INTEGER,"
            + KEY_FILEPATH + " TEXT" + ")";
    /////////// String for faqih table //////////////
    String CREATE_FAQIH_TABLE = "CREATE TABLE " + TABLE_FAQIH + "("
            + KEY_FAQIHID + " INTEGER PRIMARY KEY," + KEY_FAQIHNAME + " TEXT,"  + KEY_UPDATE + " INTEGER,"
            + KEY_FILEPATH + " TEXT" + ")";
    /////////// String for param table //////////////
    String CREATE_PARAM_TABLE = "CREATE TABLE " + TABLE_PARAM + "("
            + KEY_PARAMID + " INTEGER PRIMARY KEY," + KEY_PARAMNAME + " TEXT,"  + KEY_PARAMVALUE + " TEXT)";

	public DBH_Main(Context context) {
		super(context, DATABASE_NAME, null, DATABASE_VERSION);
	}

	@Override
	public void onCreate(SQLiteDatabase db) {
		db.execSQL(CREATE_ITEM_TABLE);
        db.execSQL(CREATE_BOOK_TABLE);
        db.execSQL(CREATE_CHAPTER_TABLE);
        db.execSQL(CREATE_FAQIH_TABLE);
        db.execSQL(CREATE_PARAM_TABLE);
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		db.execSQL("DROP TABLE IF EXISTS " + TABLE_ITEMS);
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_BOOKS);
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_CHAPTERS);
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_FAQIH);
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_PARAM);
		onCreate(db);
	}

    ///////// Insert Into Item Table ///////////
	public void insert_item(MainVO orderItem) {
		SQLiteDatabase db = this.getWritableDatabase();
		ContentValues values = new ContentValues();
		values.put(KEY_ITEMID, orderItem.getItemID());
		values.put(KEY_ITEMDESC, orderItem.getDesc());
		values.put(KEY_UPDATE, orderItem.getUpdate_time());
		values.put(KEY_FILEPATH, orderItem.getFilePath());
		// Inserting Row
		db.insert(TABLE_ITEMS, null, values);
		//db.close(); // Closing database connection
	}

    ///////// Insert Into Book Table ///////////
    public void insert_book(MainVO orderItem) {
        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues values = new ContentValues();
        values.put(KEY_BOOKID, orderItem.getBookID());
        values.put(KEY_BOOKDESC, orderItem.getDesc());
        values.put(KEY_UPDATE, orderItem.getUpdate_time());
        values.put(KEY_FILEPATH, orderItem.getFilePath());
        // Inserting Row
        db.insert(TABLE_BOOKS, null, values);
    }

    ///////// Insert Into chapter Table ///////////
    public void insert_chapter(MainVO orderItem) {
        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues values = new ContentValues();
        values.put(KEY_CHAPTERID, orderItem.getChapterID());
        values.put(KEY_BOOKID, orderItem.getBookID());
        values.put(KEY_CHAPTERDESC, orderItem.getDesc());
        values.put(KEY_UPDATE, orderItem.getUpdate_time());
        values.put(KEY_FILEPATH, orderItem.getFilePath());
        // Inserting Row
        db.insert(TABLE_CHAPTERS, null, values);
    }

    ///////// Insert Into Faqih Table ///////////
    public void insert_faqih(MainVO orderItem) {
        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues values = new ContentValues();
        values.put(KEY_FAQIHID, orderItem.getFaqihID());
        values.put(KEY_FAQIHNAME, orderItem.getDesc());
        values.put(KEY_UPDATE, orderItem.getUpdate_time());
        values.put(KEY_FILEPATH, orderItem.getFilePath());
        // Inserting Row
        db.insert(TABLE_FAQIH, null, values);
    }

    ///////// Insert Into Param Table ///////////
    public void insert_param(MainVO orderItem) {
        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues values = new ContentValues();
        values.put(KEY_PARAMID, orderItem.getParamID());
        values.put(KEY_PARAMNAME, orderItem.getDesc());
        values.put(KEY_PARAMVALUE, orderItem.getParamValue());
        // Inserting Row
        db.insert(TABLE_PARAM, null, values);
    }

	// Getting All Items
	public ArrayList<MainVO> getAllItems() {
        MainVO item;
        ArrayList<MainVO> itemList = new ArrayList<MainVO>();
		String selectQuery = "SELECT * FROM " + TABLE_ITEMS;
		SQLiteDatabase db = this.getWritableDatabase();
		Cursor cursor = db.rawQuery(selectQuery, null);
		Log.d("curser size", "" + cursor.getCount());
		if (cursor.moveToFirst()) {
			do {
                item = new MainVO();
                item.setItemID(cursor.getInt(0));
                item.setDesc(cursor.getString(1));
                item.setUpdate_time(cursor.getInt(2));
                item.setFilePath(cursor.getString(3));
                itemList.add(item);
			} while (cursor.moveToNext());
		}
		return itemList;
	}

    // Getting All Books
    public ArrayList<MainVO> getAllBooks() {
        MainVO book;
        ArrayList<MainVO> bookList = new ArrayList<MainVO>();
        String selectQuery = "SELECT * FROM " + TABLE_BOOKS;
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(selectQuery, null);
        Log.d("curser size", "" + cursor.getCount());
        if (cursor.moveToFirst()) {
            do {
                book = new MainVO();
                book.setBookID(cursor.getInt(0));
                book.setDesc(cursor.getString(1));
                book.setUpdate_time(cursor.getInt(2));
                book.setFilePath(cursor.getString(3));
                bookList.add(book);
            } while (cursor.moveToNext());
        }
        return bookList;
    }

    // Getting Chapters for a Book
    public ArrayList<MainVO> getBookChapters(final String book_id) {
        MainVO chapter;
        ArrayList<MainVO> chapterList = new ArrayList<MainVO>();
        String selectQuery = "SELECT * FROM " + TABLE_CHAPTERS + " WHERE book_id = "+book_id;
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(selectQuery, null);
        Log.d("curser size", "" + cursor.getCount());
        if (cursor.moveToFirst()) {
            do {
                chapter = new MainVO();
                chapter.setChapterID(cursor.getInt(0));
                chapter.setBookID(cursor.getInt(1));
                chapter.setDesc(cursor.getString(2));
                chapter.setUpdate_time(cursor.getInt(3));
                chapter.setFilePath(cursor.getString(4));
                chapterList.add(chapter);
            } while (cursor.moveToNext());
        }
        return chapterList;
    }

    // Getting All Faqih
    public ArrayList<MainVO> getAllFaqih() {
        MainVO faqih;
        ArrayList<MainVO> faqihList = new ArrayList<MainVO>();
        String selectQuery = "SELECT * FROM " + TABLE_FAQIH;
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(selectQuery, null);
        Log.d("curser size", "" + cursor.getCount());
        if (cursor.moveToFirst()) {
            do {
                faqih = new MainVO();
                faqih.setFaqihID(cursor.getInt(0));
                faqih.setDesc(cursor.getString(1));
                faqih.setUpdate_time(cursor.getInt(2));
                faqih.setFilePath(cursor.getString(3));
                faqihList.add(faqih);
            } while (cursor.moveToNext());
        }
        return faqihList;
    }

    // Getting All Param
    public ArrayList<MainVO> getAllParam() {
        MainVO param;
        ArrayList<MainVO> paramList = new ArrayList<MainVO>();
        String selectQuery = "SELECT * FROM " + TABLE_PARAM;
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(selectQuery, null);
        //Log.d("curser size", "" + cursor.getCount());
        if (cursor.moveToFirst()) {
            do {
                param = new MainVO();
                param.setParamID(cursor.getInt(0));
                param.setDesc(cursor.getString(1));
                param.setParamValue(cursor.getString(2));
                paramList.add(param);
            } while (cursor.moveToNext());
        }
        return paramList;
    }

    // Getting Param By its name
    public MainVO getParamByName(String name) {
        MainVO param = new MainVO();
        //ArrayList<MainVO> paramList = new ArrayList<MainVO>();
        String selectQuery = "SELECT * FROM " + TABLE_PARAM + " WHERE "+KEY_PARAMNAME+" = '"+name+"'";
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(selectQuery, null);
        if (cursor.moveToFirst()) {
            do {
                param.setParamID(cursor.getInt(0));
                param.setDesc(cursor.getString(1));
                param.setParamValue(cursor.getString(2));
                //paramList.add(param);
            } while (cursor.moveToNext());
        }
        if (param.equals(null))
            param = new MainVO();
        return param;
    }



	public void deleteAllItems(){
		SQLiteDatabase db = this.getWritableDatabase();
		db.execSQL("DELETE FROM " + TABLE_ITEMS);
		db.close();
	}

    public void deleteAllBooks(){
        SQLiteDatabase db = this.getWritableDatabase();
        db.execSQL("DELETE FROM "+TABLE_BOOKS);
        db.close();
    }

    public void deleteAllChapters(){
        SQLiteDatabase db = this.getWritableDatabase();
        db.execSQL("DELETE FROM "+TABLE_CHAPTERS);
        db.close();
    }

    public void deleteAllFaqih(){
        SQLiteDatabase db = this.getWritableDatabase();
        db.execSQL("DELETE FROM "+TABLE_FAQIH);
        db.close();
    }

    public void deleteAllParam(){
        SQLiteDatabase db = this.getWritableDatabase();
        db.execSQL("DELETE FROM "+TABLE_PARAM);
        db.close();
    }

}
